/* ReSCAL - regenesis
 *
 * Copyright (C) 2011
 *
 * Author: Olivier Rozier <rozier@ipgp.fr>
 *
 * This file is part of ReSCAL.
 *
 * ReSCAL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "regenesis.h"

#define PI 3.14159265

int H0, L0, P0, HL0, HP0, HLP0;       // les dimensions de la terre initiale
int H, L, P, HL, HP, HLP;       // les dimensions de la terre finale
int Hd2, Ld2, Hd3, Hd4;
Cell  *TE=NULL;	           // notre 'terre'
Cell  *TE0=NULL;	           // ancienne 'terre'
#ifdef MODELE_DUN
float alpha;	// angle de rotation
#endif

double drand48();
void	srand48();


void read_terre(char *nom)
{
	FILE *fp;

  printf("allocation terre TE0 : %d\n", (int)sizeof(Cell) * HLP0);fflush(stdout);
	TE0 = (Cell *) malloc( sizeof(Cell) * HLP0);
	if( TE0 == NULL){
		printf("allocation de la terre impossible !\n");fflush(stdout);
		exit(-1);
	}

  fp = fopen(nom,"r");
  if ( ! fp ){
 	  printf("erreur ouverture fichier terre binaire\n");fflush(stdout);
 	  exit(-4);
  }

  printf("lecture terre TE0 : %d x %d x %d \n" , H0, L0, P0);

  fread(TE0, sizeof(char), HLP0, fp);

  fclose(fp);
}

// on copie TE dans TE0
void cp_TE_TE0()
{
  	if (HLP != HLP0){
          TE0 = (Cell *) realloc((void *)TE0,  sizeof(Cell) * HLP);

          if( TE0 == NULL){
            printf("reallocation de la terre impossible !\n");fflush(stdout);
            exit(-1);
          }
          H0 = H;
	  L0 = L;
	  P0 = P;
	  HL0 = HL;
	  HLP0 = HLP;
	}

	memcpy( (void *)TE0, (void *)TE, sizeof(Cell) * HLP); //TE0 <- TE
}

void change_cel()
// in : TE
// out : TE
{
  int i, j, k, n;
  float di, dj, dk;
  int Ld2, Pd2, Hd2;
  float Ldx, Ldy, Ldz;
  Cell *aux;
  
    Ld2 = (int) L/2;
    Pd2 = (int) P/2;
    Hd2 = (int) H/2;
    int lc = L*0.3;//Ld2; //L*0.75; //Ld2+40; //L/4; //Ld2; //largeur couloir
    Ldx= Ld2-0.5; //Ld3;//Ld4;
    Ldy= Pd2-0.5; //Ld4; //(int) L/2;
    Ldz= Hd2;
    int rcyl = Ld2-5; //rayon cylindre
	
    printf("change_cel\n");

	aux = TE;
	for(k=0,n=1; k < P; k++,n=1-n) // profondeur
	  for(j=0; j < H; j++) // hauteur
	    for(i=0; i < L; i++, aux++){ //largeur
             di = i - Ldx;
             dj = j - Ldz;
             dk = k - Ldy;
               //if ((n==0) || (j > H0) ||  (j==0) || (j-0.08*k < 0))
			   //aux->celldata   = DUM;
			   //else if ((n==1))
				//aux->celldata   = GR;
			  //if (aux->celldata == GR) aux->celldata = DUM; //dune en beton
              //if (aux->celldata == GRJ) aux->celldata = EAUC; //dune en beton
			  //if (aux->celldata == IN) aux->celldata = DUM; //dune en beton
			  //if ((aux->celldata == EAUT) || (aux->celldata == EAUC)) aux->celldata = (drand48() < 0.08) ? EAUC : EAUT; //antidunes
              //if (aux->celldata == EAUT) aux->celldata = EAUC; //pas de turbulence
              //if ((i<40) && (j==H-8)) aux->celldata=EAUC; //lissage
              //if (k!=Ld2) aux->celldata = DUM; //couloir de largeur 1 -> modele 2d
              //if (aux->celldata == DUM) aux->celldata = EAUC; //pas de DUM
              //if (aux->celldata == IN) aux->celldata = (k==Ld2)? EAUC : DUM; //pas de IN
              //if (aux->celldata == IN) aux->celldata = DUM; //pas de IN
              //if ((j>=1) && (j<=H-8) && (aux->celldata == DUM)) aux->celldata = EAUC; //on enleve le couloir, mais on garde le sol et le plafond
              //if ((k<Ld2-lc/2) || (k>Ld2+lc/2)) aux->celldata = DUM; //couloir de largeur lc
	      //if ((i==5) && (j==0) && (k==Ld2)) aux->celldata = IN; //source lineaire
	      //if ((i==5) && (j==0) && (k>=Ld2-60) && (k<=Ld2+60)) aux->celldata = IN; //source lineaire dans le couloir
          //if ((i==5) && (j==0) && (k >= Ld2-lc) && (k <= Ld2+lc))//source lineaire dans le couloir
	          //if ((i==5) && (j==0)) aux->celldata = IN;
		  //if ((j==1) && (aux->celldata == IN)) aux->celldata = EAUC;
	      //if ((i==0) && (aux->celldata == GR)) aux->celldata = EAUC; //pas de grain contre le bord, SVP ...
	      if ((j == 0) && ((di*di + dk*dk >= rcyl*rcyl) && ((di+1)*(di+1) + dk*dk < rcyl*rcyl))) ////source de grains en demi-cercle
                    aux->celldata = IN;
	          if ((j ==H-1) && (i > Ld2) && (di*di + dk*dk >= rcyl*rcyl)) ////sortie de grains en cercle
                    aux->celldata = OUT;
	    }
}

#ifdef MODELE_DUN
void rotation()
// in : TE0
// out : TE
{
  int i, j, k;
  int di, dk, i0, k0;
  Cell *aux;

  //alpha = PI;
  float co = cos(alpha);
  float si = sin(alpha);

	printf("rotation de %f degres\n", alpha*180/PI);

	aux = TE;
	for(k=0; k < P; k++) // profondeur
	  for(j=0; j < H; j++) // hauteur
	    for(i=0; i < L; i++, aux++){ //largeur
              di = i - Ld2;
              dk = k - Ld2;
              if (di*di + dk*dk < Ld2*Ld2){
                i0 = Ld2 + co*di - si*dk;
                k0 = Ld2 + si*di + co*dk;
                aux->celldata = (TE0+i0+j*L0+k0*HL0)->celldata;
              }
	      else
	        aux->celldata   = (TE0+i+j*L0+k*HL0)->celldata;
              //if (aux->celldata == DUM) aux->celldata = EAUC;
            }
}
#endif

void rotation90(int n)
// in : TE0
// out : TE
{
  Cell *aux;
  int i, j, k;
  int i0, k0;
  float a = n*PI/2;
  float co = cos(a);
  float si = sin(a);
  float di0 = Ld2 - 0.5;
  float dk0 = Ld2 - 0.5;
  float di, dk;

  printf("rotation complete de %f degres\n", a*180/PI);

  aux = TE;
  for(k=0; k < P; k++) // profondeur
    for(j=0; j < H; j++) // hauteur
      for(i=0; i < L; i++, aux++){ //largeur
        di = i - di0;
        dk = k - dk0;
        i0 = di0 + co*(float)di - si*(float)dk;
        k0 = dk0 + si*(float)di + co*(float)dk;
        aux->celldata = (TE0+i0+j*L0+k0*HL0)->celldata;
      }
}

void translation()
// in : TE
// out : TE
{
  int i, j, k;
  Cell *aux;

	//translation est-ouest
	int d_eo=550; //L/3; //L/4; //L/10; //L/5;
	aux = TE;
	printf("translation : %d\n", d_eo);
	for(k=0; k < P; k++) // profondeur
	  for(j=0; j < H; j++) // hauteur
	    for(i=0; i < L; i++, aux++){ //largeur
#ifdef MODELE_DUN
              //aux->celldata = (i<L-d_eo)? (aux+d_eo)->celldata : (aux->celldata == DUM)? DUM : EAUC;
              //if ((i>=L-d_eo) && (j==H-2)) aux->celldata = (drand48() < 0.2) ? GR : EAUC; //sable
              //if (i>=L-d_eo) aux->celldata = DUM;//(j<=H-3)? EAUC : DUM;
              if (i < L-d_eo){
			    if (aux->celldata != IN) aux->celldata = (aux+d_eo)->celldata;
			  }
              else{
                //if (aux->celldata != DUM && (aux+d_eo)->celldata != DUM) aux->celldata = (aux+d_eo-L)->celldata;//EAUC;
				//if (aux->celldata != DUM) aux->celldata = (aux+d_eo)->celldata;
				if (aux->celldata != DUM ) aux->celldata = EAUC;
              }
#endif
            }
}

void translation_alldir()
// in : TE0
// out : TE
{
  int i, j, k;
  Cell *aux;

	//translation est-ouest
	int d_eo=100; //L/3; //L/4; //L/10; //L/5;
	//translation nord-sud
	int d_ns=100;
	aux = TE;
	printf("translation : %d , %d\n", d_eo, d_ns);
	for(k=0; k < P; k++) // profondeur
	  for(j=0; j < H; j++) // hauteur
	    for(i=0; i < L; i++, aux++){ //largeur
#ifdef MODELE_DUN
            if ((i+d_eo >= 0) && (i+d_eo < L) && (k+d_ns >= 0) && (k+d_ns < P)){
            if (aux->celldata != IN) aux->celldata = (TE0+d_eo+i+j*L+(k+d_ns)*HL)->celldata;
			  }
              else{
			if (aux->celldata != DUM ) aux->celldata = EAUC;
             }
#endif
            }
}

void translation_cyclic()
// in : TE0
// out : TE
{
  int i, j, k;
  Cell *aux;

	//translation est-ouest
	int d_eo=140; //L/3; //L/4; //L/10; //L/5;
	aux = TE;
	printf("translation : %d\n", d_eo);
	for(k=0; k < P; k++) // profondeur
	  for(j=0; j < H; j++) // hauteur
	    for(i=0; i < L; i++, aux++){ //largeur
#ifdef MODELE_DUN
              //aux->celldata = (i<L-d_eo)? (aux+d_eo)->celldata : (aux->celldata == DUM)? DUM : EAUC;
              //if ((i>=L-d_eo) && (j==H-2)) aux->celldata = (drand48() < 0.2) ? GR : EAUC; //sable
              //if (i>=L-d_eo) aux->celldata = DUM;//(j<=H-3)? EAUC : DUM;
              if (i < L-d_eo){
			    if (aux->celldata != IN) aux->celldata = (TE0+d_eo+i+j*L+k*HL)->celldata;
			  }
              else{
                //if (aux->celldata != DUM && (aux+d_eo-L)->celldata != DUM) aux->celldata = (TE0+d_eo-L+i+j*L+k*HL)->celldata;//EAUC;
                if (aux->celldata != DUM) aux->celldata = (TE0+d_eo-L+i+j*L+k*HL)->celldata;//EAUC;
				//if (aux->celldata != DUM) aux->celldata = (aux+d_eo)->celldata;
				//if (aux->celldata != DUM ) aux->celldata = EAUC;
              }
#endif
            }
}

void translation_cyclic_ns()
// in : TE0
// out : TE
{
  int i, j, k;
  Cell *aux;

	//translation nord_sud
	int d_ns=220; //L/3; //L/4; //L/10; //L/5;
	aux = TE;
	printf("translation : %d\n", d_ns);
	for(k=0; k < P; k++) // profondeur
	  for(j=0; j < H; j++) // hauteur
	    for(i=0; i < L; i++, aux++){ //largeur
#ifdef MODELE_DUN
              //aux->celldata = (i<L-d_eo)? (aux+d_eo)->celldata : (aux->celldata == DUM)? DUM : EAUC;
              //if ((i>=L-d_eo) && (j==H-2)) aux->celldata = (drand48() < 0.2) ? GR : EAUC; //sable
              //if (i>=L-d_eo) aux->celldata = DUM;//(j<=H-3)? EAUC : DUM;
              if (k < P-d_ns){
			    if (aux->celldata != IN) aux->celldata = (TE0+i+j*L+(k+d_ns)*HL)->celldata;
			  }
              else{
                //if (aux->celldata != DUM && (aux+d_eo-L)->celldata != DUM) aux->celldata = (TE0+d_eo-L+i+j*L+k*HL)->celldata;//EAUC;
                if (aux->celldata != DUM) aux->celldata = (TE0+i+j*L+(d_ns-P+k)*HL)->celldata;//EAUC;
				//if (aux->celldata != DUM) aux->celldata = (aux+d_eo)->celldata;
				//if (aux->celldata != DUM ) aux->celldata = EAUC;
              }
#endif
            }
}
void transcale1()
// in : TE0
// out : TE
{
  int i, j, k;
  Cell *aux;

	//premiere translation/homothetie vers l'ouest

	int d_eo=L/10; //L/8;
        int ii, jj, kk;
	int d_ns=0;
	float coef1 = 0.4;
	aux = TE;
	printf("translation 1 + rescaling\n");
	for(k=0; k < P; k++) // profondeur
	  for(j=0; j < H; j++) // hauteur
	    for(i=0; i < L; i++, aux++){ //largeur
              ii = (i*coef1)+d_eo;
              jj = H-1-((H-1-j)*coef1);
              kk = Ld2+((k-Ld2)*coef1)+d_ns;
#ifdef MODELE_DUN
              aux->celldata = ((ii>0) && (ii<L) && (jj>0) && (jj<H) && (kk>0) && (kk<L))? (TE0+ii+jj*L+kk*HL)->celldata : EAUC;
	      //if ((j<H-1) && (aux->celldata == DUM)) aux->celldata = EAUC;
	      if (j == 0)
			 aux->celldata = DUM;
		  //else if ( j == H-1) aux->celldata = EAUC;
#endif
             }
}

void transcale2()
// in : TE0
// out : TE
{
  int i, j, k;
  Cell *aux;

	//deuxieme translation/homothetie vers le nord-ouest

	int d_eo=L/8; //L/8;
        int ii, jj, kk;
	//int d_ns=2*L/15; //position 1
	//int d_ns=1.7*L/15;
	int d_ns=L/15; //position 2
	//int d_ns=0; //position 3
	float coef2 = 1.5;
	aux = TE;
	printf("translation 2 + rescaling\n");
        printf("d_ns = %d\n", d_ns);
	for(k=0; k < P; k++) // profondeur
	  for(j=0; j < H; j++) // hauteur
	    for(i=0; i < L; i++, aux++){ //largeur
              ii = (i*coef2)+d_eo;
              jj = H-1-((H-1-j)*coef2);
              kk = Ld2+((k-Ld2)*coef2)+d_ns;
#ifdef MODELE_DUN
              if (aux->celldata != GR) aux->celldata = ((ii>0) && (ii<L) && (jj>0) && (kk>0) && (kk<L))? (TE0+ii+jj*L+kk*HL)->celldata : EAUC;
	      if ((j<H-1) && (aux->celldata == DUM)) aux->celldata = EAUC;
	      else if (j==H-1) aux->celldata = DUM;
#endif
           }
}

void change_height()
// in : TE0
// out : TE
{
  int i, j, k;
  Cell *aux;

	//redimensionnement
	int H0 = H;
	int HL0 = HL;
    //int HLP0 = HLP;
        H = 200; //H0*2/3; //H0*2;
	    Hd2 = H/2;
        HL = H*L;
        HLP = HL*P;

        printf("reallocation terre : %d\n", (int)sizeof(Cell) * HLP);fflush(stdout);
	TE = (Cell *) realloc((void *)TE,  sizeof(Cell) * HLP);
	if( TE == NULL){
		printf("reallocation de la terre impossible !\n");fflush(stdout);
		exit(-1);
	}
	aux = TE;

	printf("redimensionnement vertical : %d x %d x %d \n", H, L, P);

	for(k=0; k < P; k++) // profondeur
	  for(j=0; j < H; j++) // hauteur
	    for(i=0; i < L; i++, aux++){ //largeur
#ifdef MODELE_DUN
	      //aux->celldata   = (H0-H+j >= 0) ? (TE0+i+(H0-H+j)*L+k*HL0)->celldata : (drand48() < 0.92) ? EAUC : EAUT;
	      //aux->celldata   = (H0-H+j >= 0) ? (TE0+i+(H0-H+j)*L+k*HL0)->celldata : (TE0+i+k*HL0+L)->celldata == DUM ? DUM : (drand48() < 0.98) ? EAUC : EAUT;
	      aux->celldata   = (H0-H+j > 0) ? (TE0+i+(H0-H+j)*L+k*HL0)->celldata : (TE0+i+k*HL0+L)->celldata == DUM ? DUM : EAUC;
	      //if ((aux->celldata == EAUT) || (aux->celldata == EAUC)) aux->celldata = (drand48() < 0.98) ? EAUC : EAUT;
              if (j == 0) aux->celldata = DUM; //plafond
#else
	      aux->celldata   = (H0-H+j >= 0) ? (TE0+i+(H0-H+j)*L+k*HL0)->celldata : MOINS;
#endif
              //if (j==H-1) aux->celldata = DUM; //sol plat
            }

}


void add_layer()
// in : TE0
// out : TE
{
  int i, j, k;
  Cell *aux;

          //redimensionnement
        H = H0+40; //1 couche en haut + 7 couches en bas
	    Hd2 = H/2;
        HL = H*L;
        HLP = HL*P;

        printf("reallocation terre : %d\n", (int)sizeof(Cell) * HLP);fflush(stdout);
        TE = (Cell *) realloc((void *)TE,  sizeof(Cell) * HLP);
        if( TE == NULL){
                printf("reallocation de la terre impossible !\n");fflush(stdout);
                exit(-1);
        }
        aux = TE;

        printf("redimensionnement vertical : %d x %d x %d \n", H, L, P);

        for(k=0; k < P; k++) // profondeur
          for(j=0; j < H; j++) // hauteur
            for(i=0; i < L; i++, aux++){ //largeur
              //aux->celldata   = ((j < 1) && (j >= H-40)) ? (TE0+i+(j-1)*L+k*HL0)->celldata : EAUC;
			  //aux->celldata   = ((j >= 1) && (j <= H-50)) ? (TE0+i+(j-1)*L+k*HL0)->celldata : EAUC;
			  aux->celldata   = ((j >= H+40)) ? (TE0+i+(j-1)*L+k*HL0)->celldata : EAUC;
            }
}

void change_width()
// in : TE0
// out : TE
{
  int i, j, k;
  Cell *aux;

        //redimensionnement horizontal est-ouest
	int d;
    L = L0*2; //L0*2;
	Ld2 = L/2;
    HL = H*L;
    HLP = HL*P;
	d = (L-L0)/2;

        printf("reallocation terre : %d\n", (int)sizeof(Cell) * HLP);fflush(stdout);
        TE = (Cell *) realloc((void *)TE,  sizeof(Cell) * HLP);
        if( TE == NULL){
                printf("reallocation de la terre impossible !\n");fflush(stdout);
                exit(-1);
        }
        aux = TE;

        printf("redimensionnement horizontal est-ouest : %d x %d x %d \n", H, L, P);

        for(k=0; k < P; k++){ // profondeur
          for(j=0; j < H; j++){ // hauteur
            for(i=0; i < L; i++, aux++){ //largeur
	          if (i<d)
		        aux->celldata = (TE0+j*L0+k*HL0) ->celldata;
	          else if (i>=L-d)
		        aux->celldata = (TE0+(L0-1)+j*L0+k*HL0) ->celldata;
              else
	            aux->celldata = (TE0+(i-d)+j*L0+k*HL0)->celldata;
	          }
          }
      }
		//if ((aux->celldata == EAUT) || (aux->celldata == EAUC)) aux->celldata = (drand48() < 0.98) ? EAUC : EAUT;
              //if (j==H-1) aux->celldata = DUM; //sol plat
            }


void change_depth()
// in : TE0
// out : TE
{
  int i, j, k, d;
  Cell *aux;
  
  //redimensionnement horizontal nord-sud centre
  
  P = 600;
  d = (P-P0)/2;
  HLP = HL*P;
        
        printf("reallocation terre : %d\n", (int)sizeof(Cell) * HLP);fflush(stdout);
        TE = (Cell *) realloc((void *)TE,  sizeof(Cell) * HLP);
        if( TE == NULL){
                printf("reallocation de la terre impossible !\n");fflush(stdout);
                exit(-1);
        }
        aux = TE;

        printf("redimensionnement horizontal nord-sud : %d x %d x %d \n", H, L, P);
        
        for(k=0; k < P; k++){ // profondeur
          for(j=0; j < H; j++){ // hauteur
            for(i=0; i < L; i++, aux++){ //largeur
              if (k<d)
                aux->celldata = (TE0+i+j*L0)->celldata;
              else if (k>=P-d)
                aux->celldata = (TE0+i+j*L0+(P0-1)*HL0)->celldata;
              else
                aux->celldata = (TE0+i+j*L0+(k-d)*HL0)->celldata;
            }
          }
        }
}

void regenere_terre()
{
  	L=L0;
  	H=H0;
  	P=P0;
  	HL=HL0;
  	HLP=HLP0;

	Hd2 = (int) H0/2;
    Ld2 = (int) L0/2;
	Hd3 = (int) 2*H0/3;
	Hd4 = (int) H0/4;


    printf("allocation terre TE : %d\n", (int)sizeof(Cell) * HLP);fflush(stdout);
	TE = (Cell *) malloc( sizeof(Cell) * HLP);
	if( TE == NULL){
		printf("allocation de la terre impossible !\n");fflush(stdout);
		exit(-1);
	}

  	printf("nouvelle terre TE : %d x %d x %d \n" , H, L, P);

#ifdef MODELE_DUN
	// Rotation d'angle alpha avec table tournante ( ne pas l'enlever )
	rotation();
#endif

	// Rotation complete d'angle n*90 degres
	//cp_TE_TE0();
	//rotation90(1);

	// Translation d'ouest en est
	//translation();

	// Translation dans toute direction
	//cp_TE_TE0();
	//translation_alldir();

	// Translation d'ouest en est, avec bouclage (conservation des grains)
	//cp_TE_TE0();
	//translation_cyclic();

    // Translation d'ouest en est, avec bouclage (conservation des grains)
	//cp_TE_TE0();
	//translation_cyclic_ns();     

	// Translation/homothetie vers l'ouest
	//cp_TE_TE0();
	//transcale1();

	// Translation, homothetie vers le nord-ouest et fusion avec l'ancienne TERRE
	//cp_TE_TE0();
	//transcale2();

	// Augmentation de la hauteur H
	//cp_TE_TE0();
	//change_height();

	// Augmentation de la longueur L
	//cp_TE_TE0();
	//change_width();

	// Augmentation de la profondeur P
	//cp_TE_TE0();
	//change_depth();

	// Ajout de couches horizontales en haut et en bas
	//cp_TE_TE0();
	//add_layer();

	// Modification du type des cellules
	change_cel();

}

void dump_terre()
{
	FILE *fp;
	char nom[256];
	int i,j;
	Cell *pt;
	char *buf,*aux;

  // fichier binaire
	sprintf(nom,"TERRE_%d_%d_%d.bin",H,L,P);
  fp = fopen(nom,"w");
  if ( ! fp ){
 	  printf("erreur ouverture fichier dump terre binaire\n");fflush(stdout);
 	  exit(-4);
  }

  buf = (char*)malloc(sizeof(char)*HL);
  pt = TE;
  for(j=0; j<P; j++){
    aux = buf;
    for(i=0; i<HL; i++, pt++){
      *aux++ = (char) pt->celldata;
    }
 	  fwrite(buf,sizeof(char),HL,fp);
  }
 	fclose(fp);
  free(buf);
}


void usage()
{
#ifdef MODELE_DUN
  printf("DUNES model\n");
  printf("usage: regenesis <binary file> H L D angle [-g <n>]\n");
#else
  printf("usage: regenesis <binary file> H L D [-g <n>]\n");
#endif
  printf("  H: height\n");
  printf("  L: lenght\n");
  printf("  D: depth\n");
#ifdef MODELE_DUN
  printf("  angle: rotation angle (degrees)\n");
#endif
  printf("  -g <n>: random seed\n");
  exit(-1);
}

int main(int argc, char **argv)
{
  int graine = 68374;
  int n, nmin;
  char *filename;

#ifdef MODELE_DUN
  nmin = 6;
#else
  nmin = 5;
#endif

  if (argc < nmin)
    usage();

  filename = argv[1];
  H0 = atoi(argv[2]);
  L0 = atoi(argv[3]);
  P0 = atoi(argv[4]);
#ifdef MODELE_DUN
  alpha = atoi(argv[5]);
  alpha = (alpha/180)*PI;
#endif

  HL0 = H0*L0;
  HLP0 = HL0*P0;
  HP0 = H0*P0;

  for(n=nmin; n<argc; n++){
    if (!strcmp(argv[n],"-g")){
      graine = atoi(argv[++n]);
    }
  }

  printf("regenesis\n");fflush(stdout);

#ifdef MODELE_DUN
  printf("modele DUNES\n");fflush(stdout);
#elif defined(MODELE_RIV)
  printf("modele RIV\n");fflush(stdout);
#endif

  srand48(graine);

  read_terre(filename);

  regenere_terre();

  dump_terre();

  printf("regenesis : fin\n");fflush(stdout);

  return 1;
}

